#ifndef WIT_EVENT_H
#define WIT_EVENT_H

#include "wit.h"

namespace wit {

class Hotspot;

class Event {
public:
    enum Type { 
        Joystick,       /**< The event was generated by changing the position of an analog joystick. */
        Analog,         /**< The event was generated by changing the pressure on an analog button. */
        Motion,         /**< The event was generated by the motion sensor of a Wii Remote or Nunchuk. */
        Pointer,        /**< The event was generated by the infrared sensor of a Wii Remote. */
        PointerLeave,   /**< The event was generated by the pointer moving out of a hotspot. */
        ButtonPress,    /**< The event was generated by pressing a non-analog button or by fully depressing an analog button. */
        ButtonRelease,  /**< The event was generated by releasing a non-analog button or by releasing a fully-depressed analog button, */
        User = 256      /**< Base value for user-defined events. */
    };

    int type() const;

protected:
    Event(int type);
    
private:
    int m_type;
};

class InputEvent : public Event {
public:
    Controller controllerType;
    unsigned short controllerNumber;

protected:
    InputEvent(int type, Controller ctype, unsigned short id);
};

class ButtonEvent : public InputEvent {
public:
    ButtonEvent(Event::Type type, Controller ctype, unsigned short id, int button = Button_None);

    Button button;
};

class JoystickEvent : public InputEvent {
public:
    JoystickEvent(Controller ctype, unsigned short id, bool isRightStick = false);

    void setByPosition(signed char px, signed char py);
    void setByPosition(float px, float py);
    void setByAngle(float pangle, float pmag);

    bool isRightStick;
    float x, y, angle, magnitude;
};

class AnalogEvent : public InputEvent {
public:
    AnalogEvent(Controller ctype, unsigned short id, AnalogButton trigger, float pressure);

    AnalogButton trigger;
    float pressure;
};

class MotionEvent : public InputEvent {
public:
    MotionEvent(Controller ctype, unsigned short id, float x, float y, float z, float pitch, float roll);

    float x, y, z, pitch, roll; // TODO: read yaw from MotionPlus when API becomes available
};

class PointerEvent : public InputEvent {
public:
    PointerEvent(unsigned short id, float x, float y, float z);

    bool isValid() const;
    float x, y, z;
};

class PointerLeaveEvent : public InputEvent {
public:
    PointerLeaveEvent(unsigned short id, Hotspot* hotspot);

    Hotspot* hotspot;
};

}

#endif
